/// <reference types="node" />
import { IRouter, NextFunction, Request, Response } from "express";
import { K8SHealthStatus } from "./enum-k8s-health-status";
import { DisposableSingletonContainer } from "./disposable-singleton-container";
declare module 'express' {
    interface Express {
        [Symbol.asyncDispose]?: () => Promise<void>;
    }
}
/**
 * The ApplicationBuilder class is responsible for building an express application.
 * It sets up the application's health status, ports, middlewares, routers, and error handling.
 *
 * @property {K8SHealthStatus} healthStatus - The health status of the application.
 * @property {string} applicationName - The name of the application.
 * @property {any} swaggerDocument - The swagger document for the application(JSON representation).
 */
export declare class ApplicationBuilder {
    applicationName: string;
    swaggerDocument: any;
    private readonly currentProcess;
    private readonly exitSignals;
    private readonly container;
    healthStatus: K8SHealthStatus;
    private applicationPort;
    private healthPort;
    private appMiddlewares;
    private appRouters;
    private helmetMiddleware;
    private bodyParserUrlEncodingMiddleware;
    private bodyParserJsonMiddleware;
    private catchAllErrorResponseTransformer;
    private readonly exitHandler;
    /**
     * Creates an instance of ApplicationBuilder.
     *
     * @param {string} applicationName - The name of the application.
     * @param {any} swaggerDocument - The swagger document for the application(JSON representation).
     * @param {NodeJS.Process} currentProcess - The current process.
     * @param {NodeJS.Signals[]} exitSignals - The exit signals.
     * @param {DisposableSingletonContainer} container - The container for disposable singletons.
     */
    constructor(applicationName?: string, swaggerDocument?: any, currentProcess?: NodeJS.Process, exitSignals?: NodeJS.Signals[], container?: DisposableSingletonContainer);
    /**
     *  Used to overrides the application port default(3000).
     * @param {number} port new application port.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideAppPort(port: number): ApplicationBuilder;
    /**
     *  Used to overrides the health port default(5678).
     * @param {number} port new health port.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideHealthPort(port: number): ApplicationBuilder;
    /**
     * Used to overrides the helmet configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} helmet new helmet configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideHelmetConfiguration(helmet: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    /**
     * Used to overrides the bodyParserUrlEncoding configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} bodyParserUrlEncoding new bodyParserUrlEncoding configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideBodyParserUrlEncodingConfiguration(bodyParserUrlEncoding: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    /**
     * Used to overrides the bodyParserJson configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} bodyParserJson new bodyParserJson configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideBodyParserJsonConfiguration(bodyParserJson: (request: Request, response: Response, next: NextFunction) => void): ApplicationBuilder;
    /**
     * Used to overrides the catchAllErrorResponseTransformer configuration.
     * @param {(request: Request, error: unknown) => unknown} transformer new catchAllErrorResponseTransformer configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideCatchAllErrorResponseTransformer(transformer: (request: Request, error: unknown) => unknown): ApplicationBuilder;
    /**
     * Used to register a middleware.
     * @param {(request: Request, response: Response, next: NextFunction) => Promise<void>} middleware middleware to be registered.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    registerApplicationMiddleware(middleware: (request: Request, response: Response, next: NextFunction) => Promise<void>): ApplicationBuilder;
    /**
     * Used to register a router.
     * @param {string} path path for the router.
     * @param {IRouter} router router to be registered.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    registerApplicationRoutes(path: string, router: IRouter): ApplicationBuilder;
    /**
     * Used to change the health status of the application.
     * @param {K8SHealthStatus} status new health status.
     */
    changeHealthStatus(status: K8SHealthStatus): void;
    /**
     * Used to start the application using the configured parameters.
     * @returns {Promise<void>} Promise that resolves when the application is started.
     */
    start(): Promise<void>;
    /**
     * Used to stop the application. This clears all the middlewares and routers.(all config is reset to default)
     * @returns {Promise<void>} Promise that resolves when the application is stopped.
     */
    [Symbol.asyncDispose](): Promise<void>;
    private appExpressListen;
    private healthExpressListen;
    private checkHealthStatus;
    private errorHandler;
}
