"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApplicationBuilder = void 0;
const express_1 = __importDefault(require("express"));
const helmet_1 = __importDefault(require("helmet"));
const bodyParser = __importStar(require("body-parser"));
const swaggerUi = __importStar(require("swagger-ui-express"));
const enum_k8s_health_status_1 = require("./enum-k8s-health-status");
const disposable_singleton_container_1 = require("./disposable-singleton-container");
/**
 * The ApplicationBuilder class is responsible for building an express application.
 * It sets up the application's health status, ports, middlewares, routers, and error handling.
 *
 * @property {K8SHealthStatus} healthStatus - The health status of the application.
 * @property {string} applicationName - The name of the application.
 * @property {any} swaggerDocument - The swagger document for the application(JSON representation).
 */
class ApplicationBuilder {
    /**
     * Creates an instance of ApplicationBuilder.
     *
     * @param {string} applicationName - The name of the application.
     * @param {any} swaggerDocument - The swagger document for the application(JSON representation).
     * @param {NodeJS.Process} currentProcess - The current process.
     * @param {NodeJS.Signals[]} exitSignals - The exit signals.
     * @param {DisposableSingletonContainer} container - The container for disposable singletons.
     */
    constructor(applicationName = 'Application', swaggerDocument = null, currentProcess = process, exitSignals = ['SIGINT', 'SIGTERM'], container = new disposable_singleton_container_1.DisposableSingletonContainer()) {
        this.applicationName = applicationName;
        this.swaggerDocument = swaggerDocument;
        this.currentProcess = currentProcess;
        this.exitSignals = exitSignals;
        this.container = container;
        this.healthStatus = enum_k8s_health_status_1.K8SHealthStatus.ALL_OK;
        this.applicationPort = 3000;
        this.healthPort = 5678;
        this.appMiddlewares = new Array();
        this.appRouters = new Map();
        this.exitHandler = this.container.disposeAll.bind(this.container);
        this.exitSignals.forEach(signal => {
            this.currentProcess.once(signal, this.exitHandler);
        });
        this.helmetMiddleware = this.container.bootstrap.createInstanceWithoutConstructor(helmet_1.default);
        this.bodyParserJsonMiddleware = this.container.bootstrap.createInstanceWithoutConstructor(bodyParser.json, [{ limit: '1mb' }]);
        this.bodyParserUrlEncodingMiddleware = this.container.bootstrap.createInstanceWithoutConstructor(bodyParser.urlencoded, [{ extended: true }]);
        this.catchAllErrorResponseTransformer = (req, error) => ({
            apistatus: 500,
            err: [{
                    errcode: 500,
                    errmsg: `Unhandled exception occured, please retry your request.`
                }]
        });
    }
    /**
     *  Used to overrides the application port default(3000).
     * @param {number} port new application port.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideAppPort(port) {
        this.applicationPort = port;
        return this;
    }
    /**
     *  Used to overrides the health port default(5678).
     * @param {number} port new health port.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideHealthPort(port) {
        this.healthPort = port;
        return this;
    }
    /**
     * Used to overrides the helmet configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} helmet new helmet configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideHelmetConfiguration(helmet) {
        this.helmetMiddleware = helmet;
        return this;
    }
    /**
     * Used to overrides the bodyParserUrlEncoding configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} bodyParserUrlEncoding new bodyParserUrlEncoding configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideBodyParserUrlEncodingConfiguration(bodyParserUrlEncoding) {
        this.bodyParserUrlEncodingMiddleware = bodyParserUrlEncoding;
        return this;
    }
    /**
     * Used to overrides the bodyParserJson configuration.
     * @param {(request: Request, response: Response, next: NextFunction) => void} bodyParserJson new bodyParserJson configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideBodyParserJsonConfiguration(bodyParserJson) {
        this.bodyParserJsonMiddleware = bodyParserJson;
        return this;
    }
    /**
     * Used to overrides the catchAllErrorResponseTransformer configuration.
     * @param {(request: Request, error: unknown) => unknown} transformer new catchAllErrorResponseTransformer configured middleware.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    overrideCatchAllErrorResponseTransformer(transformer) {
        this.catchAllErrorResponseTransformer = transformer;
        return this;
    }
    /**
     * Used to register a middleware.
     * @param {(request: Request, response: Response, next: NextFunction) => Promise<void>} middleware middleware to be registered.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    registerApplicationMiddleware(middleware) {
        this.appMiddlewares.push(middleware);
        return this;
    }
    /**
     * Used to register a router.
     * @param {string} path path for the router.
     * @param {IRouter} router router to be registered.
     * @returns {ApplicationBuilder} ApplicationBuilder instance.
     */
    registerApplicationRoutes(path, router) {
        this.appRouters.set(path, router);
        return this;
    }
    /**
     * Used to change the health status of the application.
     * @param {K8SHealthStatus} status new health status.
     */
    changeHealthStatus(status) {
        this.healthStatus = status;
    }
    /**
     * Used to start the application using the configured parameters.
     * @returns {Promise<void>} Promise that resolves when the application is started.
     */
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.container.createInstanceWithoutConstructor('applicationExpress', this.appExpressListen.bind(this));
            yield this.container.createInstanceWithoutConstructor('healthExpress', this.healthExpressListen.bind(this));
        });
    }
    /**
     * Used to stop the application. This clears all the middlewares and routers.(all config is reset to default)
     * @returns {Promise<void>} Promise that resolves when the application is stopped.
     */
    [Symbol.asyncDispose]() {
        return __awaiter(this, void 0, void 0, function* () {
            this.exitSignals.forEach(signal => {
                this.currentProcess.removeListener(signal, this.exitHandler);
            });
            yield this.container.disposeAll();
            this.appMiddlewares = [];
            this.appRouters.clear();
        });
    }
    //------Private Methods------//
    appExpressListen() {
        return __awaiter(this, void 0, void 0, function* () {
            const applicationExpressInstance = yield this.container.bootstrap.createAsyncInstanceWithoutConstructor(() => __awaiter(this, void 0, void 0, function* () { return Promise.resolve((0, express_1.default)()); }));
            const applicationHttpServer = yield new Promise((a, r) => {
                try {
                    applicationExpressInstance.use(this.helmetMiddleware);
                    applicationExpressInstance.use(this.bodyParserUrlEncodingMiddleware);
                    applicationExpressInstance.use(this.bodyParserJsonMiddleware);
                    if (this.swaggerDocument != null) {
                        applicationExpressInstance.use('/api-docs', swaggerUi.serve, swaggerUi.setup(this.swaggerDocument));
                    }
                    for (const middleware of this.appMiddlewares) {
                        applicationExpressInstance.use(middleware);
                    }
                    for (const [path, router] of this.appRouters) {
                        applicationExpressInstance.use(path, router);
                    }
                    applicationExpressInstance.use(this.errorHandler.bind(this));
                    const server = applicationExpressInstance.listen(this.applicationPort, () => { a(server); });
                }
                catch (e) {
                    r(e);
                }
            });
            applicationExpressInstance[Symbol.asyncDispose] = () => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const customDispose = () => __awaiter(this, void 0, void 0, function* () {
                    applicationHttpServer.close(e => e == null ? Promise.resolve() : Promise.reject(e));
                });
                yield (((_a = applicationHttpServer[Symbol.asyncDispose]) === null || _a === void 0 ? void 0 : _a.bind(applicationHttpServer)) || customDispose)();
            });
            return applicationExpressInstance;
        });
    }
    healthExpressListen() {
        return __awaiter(this, void 0, void 0, function* () {
            const healthExpressInstance = yield this.container.bootstrap.createAsyncInstanceWithoutConstructor(() => __awaiter(this, void 0, void 0, function* () { return Promise.resolve((0, express_1.default)()); }));
            const healthServer = yield new Promise((a, r) => {
                try {
                    healthExpressInstance.use(this.helmetMiddleware);
                    healthExpressInstance.get(`/health/startup`, (req, res) => __awaiter(this, void 0, void 0, function* () { return this.checkHealthStatus(res, `${this.applicationName} is down(${this.healthStatus.toString()}).`); }));
                    healthExpressInstance.get(`/health/readiness`, (req, res) => __awaiter(this, void 0, void 0, function* () { return this.checkHealthStatus(res, `${this.applicationName} is not available(${this.healthStatus.toString()}).`); }));
                    healthExpressInstance.get(`/health/liveliness`, (req, res) => __awaiter(this, void 0, void 0, function* () { return this.checkHealthStatus(res, `${this.applicationName} is not ready(${this.healthStatus.toString()}).`); }));
                    healthExpressInstance.use(this.errorHandler);
                    const server = healthExpressInstance.listen(this.healthPort, () => { a(server); });
                }
                catch (e) {
                    r(e);
                }
            });
            healthExpressInstance[Symbol.asyncDispose] = () => __awaiter(this, void 0, void 0, function* () {
                var _a;
                const customDispose = () => __awaiter(this, void 0, void 0, function* () {
                    healthServer.close(e => e == null ? Promise.resolve() : Promise.reject(e));
                });
                yield (((_a = healthServer[Symbol.asyncDispose]) === null || _a === void 0 ? void 0 : _a.bind(healthServer)) || customDispose)();
            });
            return healthExpressInstance;
        });
    }
    checkHealthStatus(res, failureMsg) {
        if (this.healthStatus === enum_k8s_health_status_1.K8SHealthStatus.ALL_OK) {
            res.status(200)
                .send(`Ok`);
        }
        else {
            res.status(503)
                .send(failureMsg);
        }
    }
    errorHandler(err, req, res, next) {
        if (res.headersSent) {
            return next(err);
        }
        const errorResponse = this.catchAllErrorResponseTransformer(req, err);
        res.status(500)
            .send(errorResponse);
    }
}
exports.ApplicationBuilder = ApplicationBuilder;
//# sourceMappingURL=application-builder.js.map